@@ -5,7 +5,7 @@ use strict;
 
 use Carp;
 use Scalar::Util qw(isweak weaken blessed reftype);
-use DBIx::Class::_Util qw(refcount hrefaddr);
+use DBIx::Class::_Util qw(refcount hrefaddr refdesc);
 use DBIx::Class::Optional::Dependencies;
 use Data::Dumper::Concise;
 use DBICTest::Util 'stacktrace';
@@ -21,15 +21,6 @@ my $refs_traced = 0;
 my $leaks_found = 0;
 my %reg_of_regs;
 
-# so we don't trigger stringification
-sub _describe_ref {
-  sprintf '%s%s(%s)',
-    (defined blessed $_[0]) ? blessed($_[0]) . '=' : '',
-    reftype $_[0],
-    hrefaddr $_[0],
-  ;
-}
-
 sub populate_weakregistry {
   my ($weak_registry, $target, $note) = @_;
 
@@ -65,7 +56,7 @@ sub populate_weakregistry {
     $refs_traced++;
   }
 
-  my $desc = _describe_ref($target);
+  my $desc = refdesc $target;
   $weak_registry->{$refaddr}{slot_names}{$desc} = 1;
   if ($note) {
     $note =~ s/\s*\Q$desc\E\s*//g;
@@ -109,18 +100,19 @@ sub visit_refs {
 
   my $visited_cnt = '0E0';
   for my $i (0 .. $#{$args->{refs}} ) {
-    next if isweak($args->{refs}[$i]);
+
+    next unless length ref $args->{refs}[$i]; # not-a-ref
+
+    my $addr = hrefaddr $args->{refs}[$i];
+
+    # no diving into weakregistries
+    next if $reg_of_regs{$addr};
+
+    next if $args->{seen_refs}{$addr}++;
+    $visited_cnt++;
 
     my $r = $args->{refs}[$i];
 
-    next unless length ref $r;
-
-    # no diving into weakregistries
-    next if $reg_of_regs{hrefaddr $r};
-
-    next if $args->{seen_refs}{my $dec_addr = Scalar::Util::refaddr($r)}++;
-
-    $visited_cnt++;
     $args->{action}->($r) or next;
 
     # This may end up being necessarry some day, but do not slow things
@@ -129,9 +121,10 @@ sub visit_refs {
     #  $visited_cnt += visit_refs({ %$args, refs => [ $t ] });
     #}
 
+    my $type = reftype $r;
+
     local $@;
     eval {
-      my $type = reftype $r;
       if ($type eq 'HASH') {
         $visited_cnt += visit_refs({ %$args, refs => [ map {
           ( !isweak($r->{$_}) ) ? $r->{$_} : ()
@@ -151,14 +144,104 @@ sub visit_refs {
         } scalar PadWalker::closed_over($r) ] }); # scalar due to RT#92269
       }
       1;
-    } or warn "Could not descend into @{[ _describe_ref($r) ]}: $@\n";
+    } or warn "Could not descend into @{[ refdesc $r ]}: $@\n";
   }
   $visited_cnt;
 }
 
+sub visit_namespaces {
+  my $args = { (ref $_[0]) ? %{$_[0]} : @_ };
+
+  my $visited = 1;
+
+  $args->{package} ||= '::';
+  $args->{package} = '::' if $args->{package} eq 'main';
+
+  if ( $args->{action}->($args->{package}) ) {
+
+    my $base = $args->{package};
+    $base = '' if $base eq '::';
+
+
+    $visited += visit_namespaces({ %$args, package => $_ }) for map
+      { $_ =~ /(.+?)::$/ ? "${base}::$1" : () }
+      grep
+        { $_ =~ /(?<!^main)::$/ }
+        do {  no strict 'refs'; keys %{ $base . '::'} }
+  }
+
+  return $visited;
+}
+
+# compiles a list of addresses stored as globals (possibly even catching
+# class data in the form of method closures), so we can skip them further on
+sub symtable_referenced_addresses {
+
+  my $refs_per_pkg;
+
+  my $dummy_addresslist;
+
+  my $seen_refs = {};
+  visit_namespaces(
+    action => sub {
+
+      no strict 'refs';
+
+      my $pkg = shift;
+      $pkg = '' if $pkg eq '::';
+      $pkg .= '::';
+
+      # the unless regex at the end skips some dangerous namespaces outright
+      # (but does not prevent descent)
+      $refs_per_pkg->{$pkg} += visit_refs (
+        seen_refs => $seen_refs,
+
+        # FIXME FIXME FIXME
+        # This is so damn odd - if we feed a constsub {1} (or in fact almost
+        # anything other than the actionsub below, any scalarref will show
+        # up as a leak, trapped by... something...
+        # Ideally we should be able to const this to sub{1} and just return
+        # $seen_refs (in fact it is identical to the dummy list at the end of
+        # a run here). Alas this doesn't seem to work, so punt for now...
+        action => sub { ++$dummy_addresslist->{ hrefaddr $_[0] } },
+
+        refs => [ map { my $sym = $_;
+          # *{"$pkg$sym"}{CODE} won't simply work - MRO-cached CVs are invisible there
+          ( CV_TRACING ? Class::MethodCache::get_cv("${pkg}$sym") : () ),
+
+          ( defined *{"$pkg$sym"}{SCALAR} and length ref ${"$pkg$sym"} and ! isweak( ${"$pkg$sym"} ) )
+            ? ${"$pkg$sym"} : ()
+          ,
+
+          ( map {
+            ( defined *{"$pkg$sym"}{$_} and ! isweak(defined *{"$pkg$sym"}{$_}) )
+              ? *{"$pkg$sym"}{$_}
+              : ()
+          } qw(HASH ARRAY IO GLOB) ),
+
+        } keys %$pkg ],
+      ) unless $pkg =~ /^ :: (?:
+        DB | next | B | .+? ::::ISA (?: ::CACHE ) | Class::C3
+      ) :: $/x;
+    }
+  );
+
+#  use Devel::Dwarn;
+#  Ddie [ map
+#    { { $_ => $refs_per_pkg->{$_} } }
+#    sort
+#      {$refs_per_pkg->{$a} <=> $refs_per_pkg->{$b} }
+#      keys %$refs_per_pkg
+#  ];
+
+  $seen_refs;
+}
+
 sub assert_empty_weakregistry {
   my ($weak_registry, $quiet) = @_;
 
+  Sub::Defer::undefer_all();
+
   # in case we hooked bless any extra object creation will wreak
   # havoc during the assert phase
   local *CORE::GLOBAL::bless;
@@ -166,6 +249,9 @@ sub assert_empty_weakregistry {
 
   croak 'Expecting a registry hashref' unless ref $weak_registry eq 'HASH';
 
+  defined $weak_registry->{$_}{weakref} or delete $weak_registry->{$_}
+    for keys %$weak_registry;
+
   return unless keys %$weak_registry;
 
   my $tb = eval { Test::Builder->new }
@@ -182,58 +268,25 @@ sub assert_empty_weakregistry {
       if defined $weak_registry->{$addr}{weakref} and ! isweak( $weak_registry->{$addr}{weakref} );
   }
 
-  # compile a list of refs stored as globals (possibly even catching
-  # class data in the form of method closures), so we can skip them
-  # further on
-  my ($seen_refs, $classdata_refs) = ({}, undef);
-
-  # the walk is very expensive - if we are $quiet (running in an END block)
-  # we do not really need to be too thorough
-  unless ($quiet) {
-    my ($symwalker, $symcounts);
-    $symwalker = sub {
-      no strict 'refs';
-      my $pkg = shift || '::';
-
-      # any non-weak globals are "clasdata" in all possible sense
-      #
-      # the unless regex at the end skips some dangerous namespaces outright
-      # (but does not prevent descent)
-      $symcounts->{$pkg} += visit_refs (
-        seen_refs => $seen_refs,
-        action => sub { ++$classdata_refs->{hrefaddr $_[0]} },
-        refs => [ map { my $sym = $_;
-          # *{"$pkg$sym"}{CODE} won't simply work - MRO-cached CVs are invisible there
-          ( CV_TRACING ? Class::MethodCache::get_cv("${pkg}$sym") : () ),
-
-          ( defined *{"$pkg$sym"}{SCALAR} and length ref ${"$pkg$sym"} and ! isweak( ${"$pkg$sym"} ) )
-            ? ${"$pkg$sym"} : ()
-          ,
-          ( map {
-            ( defined *{"$pkg$sym"}{$_} and ! isweak(defined *{"$pkg$sym"}{$_}) )
-              ? *{"$pkg$sym"}{$_}
-              : ()
-          } qw(HASH ARRAY IO GLOB) ),
-        } keys %$pkg ],
-      ) unless $pkg =~ /^ :: (?:
-        DB | next | B | .+? ::::ISA (?: ::CACHE ) | Class::C3
-      ) :: $/x;
-
-      $symwalker->("${pkg}$_") for grep { $_ =~ /(?<!^main)::$/ } keys %$pkg;
-    };
-
-    $symwalker->();
-
-#    use Devel::Dwarn;
-#    Ddie [ map
-#      { { $_ => $symcounts->{$_} } }
-#      sort
-#        {$symcounts->{$a} <=> $symcounts->{$b} }
-#        keys %$symcounts
-#    ];
-  }
-
-  delete $weak_registry->{$_} for keys %$classdata_refs;
+  # the symtable walk is very expensive
+  # if we are $quiet (running in an END block) we do not really need to be
+  # that thorough - can get by with only %Sub::Quote::QUOTED
+  delete $weak_registry->{$_} for $quiet
+    ? do {
+      my $refs = {};
+      visit_refs (
+        # only look at the closed over stuffs
+        refs => [ grep { length ref $_ } map { values %{$_->[2]} } grep { ref $_ eq 'ARRAY' } values %Sub::Quote::QUOTED ],
+        seen_refs => $refs,
+        action => sub { 1 },
+      );
+      keys %$refs;
+    }
+    : (
+      # full sumtable walk, starting from ::
+      keys %{ symtable_referenced_addresses() }
+    )
+  ;
 
   for my $addr (sort { $weak_registry->{$a}{display_name} cmp $weak_registry->{$b}{display_name} } keys %$weak_registry) {
 
